//+------------------------------------------------------------------+
//|                                             OrderManagerTest.mq4 |
//|                                      Copyright  2016, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|   MQL's OOP notes: Singleton, Command queue, and Template method |
//|                                 patterns in simple order manager |
//|                        https://www.mql5.com/en/blogs/post/681732 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2016, Marketeer"
#property link      "https://www.mql5.com/en/users/marketeer"
#property version   "1.0"
#property description "Simple trading robot based on Bollinger Bands\n"
#property description "Example for MQL's OOP notes: Singleton, Command queue, and Template method patterns in simple order manager"

#property strict

#define TIMER 1 // seconds between successive attempts of queue processing 

// Define input parameters

extern int    period = 10;
extern double width = 0.25;

extern double Lot = 0.01;
extern int    Deviation = 10;
extern int    Magic = 0;
extern int    RetryNumber = 2 /* small number just to demonstrate the loop */;
extern int    WaitAfterError = 1000 /* milliseconds */;

extern bool   BarByBar = false;
extern int    DelaySimulation = 600 /* seconds */;

// Include the framework

#define ASYNC_ORDERS
#include <OrderManager.mqh>

// Derive our custom callback class for notifications and create an object of it.

class CustomOrderCallback : public OrderCallback
{
  // TODO: override base methods to get new tickets or error codes
};

CustomOrderCallback cc;

// Code standard event handlers

int OnInit()
{
  OrderManager::getInstance().setDeviation(Deviation);
  OrderManager::getInstance().setMagic(Magic);
  OrderManager::getInstance().setMaxRetries(RetryNumber);
  OrderManager::getInstance().setTimeWait(WaitAfterError);
  OrderManager::getInstance().onInit();
  OrderManager::getInstance().bindCallback(&cc);
  if(!IsTesting()) EventSetTimer(TIMER);
  return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
  OrderManager::getInstance().onDeinit();
}

void OnTimer()
{
  OrderManager::getInstance().onTimer();
}

// And now the main part - trading

void OnTick()
{
  if(IsTesting()) // in the tester we can only emulate timer
  {
    static datetime lastTick;
    if(TimeCurrent() - lastTick >= TIMER)
    {
      OnTimer();
    }
    lastTick = TimeCurrent();
  }
  
  static datetime lastBarTime;

  double rate = Close[0];
  double ma = iBands(NULL, 0, period, width, 0, PRICE_CLOSE, MODE_MAIN, 0);
  double upper = iBands(NULL, 0, period, width, 0, PRICE_CLOSE, MODE_UPPER, 0);
  double lower = iBands(NULL, 0, period, width, 0, PRICE_CLOSE, MODE_LOWER, 0);
  
  if(OrdersTotal() == 0)
  {
    if((lastBarTime == Time[0]) && BarByBar) return;
    
    int cmd = (rate > upper) ? OP_SELL : ((rate < lower) ? OP_BUY : -1);
    
    if(cmd != -1)
    {
      // possible invocation
      // OrderSend(Symbol(), cmd, Lot, CURRENT_PRICE, 0, 0.0, 0.0, "");
      
      // if there is no command in the queue
      if(OrderManager::getInstance().getLastCommand() == NULL)
      {
        // example of async command with full constructor
        // OrderManager::getInstance().place(new OrderCreateCommand(cmd, Lot, Symbol(), CURRENT_PRICE, 0.0, 0.0, "", 0, true));
        
        OrderManager::getInstance().place(new OrderCreateCommandWithDelay(DelaySimulation, cmd, Lot));
      }
    }
  }
  else
  {
    if(OrderSelect(OrdersTotal() - 1, SELECT_BY_POS))
    {
      if(OrderSymbol() == Symbol())
      {
        if((OrderType() == OP_BUY && rate > ma)
        || (OrderType() == OP_SELL && rate < ma))
        {
          OrderClose(OrderTicket(), OrderLots(), CURRENT_PRICE, 0);
        }
      }
    }
  }

  lastBarTime = Time[0];
}

